%
% README for FFNN Matlab Package Version 4.1
%
% June 2003 (c) Copyright by 
% AG Medizinische Physik, Universitt Oldenburg
% 
%
% Contents:
%
% 1. Introduction to FFNN
% 2. Overview of MATLAB scripts and functions
% 3. Remark and Annotations
% 4. Copyright Notice and Disclaimer
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 1. Introduction to Feature Finding Neural Networks (FFNN)
%
% Feature Finding Neural Networks (FFNN) was first proposed by Tino Gram (1992) for the
% purpose of automatic speech recognition. Motivated by neurophysiological and psychoacoustical
% findings, the original ('primary') feature vector sequence of a given articulation undergoes
% further extraction of so-called 'sigma-pi cells'. Each product of the means of two 
% windows (usually in the time-frequency plain) results in a 'secondary' feature. The smaller
% window is only one element of the 'primary' feature matrix, while the larger window may be
% of any size in both dimension (1-5 Elements = 1-5 ERB or 10-50 ms seems sensible). For
% isolated (word) recognition the secondary features are integrated over time in order to
% obtain a time-independent set of secondary features. A linear two-layer neural network is 
% trained to estimate the class-dependencies. The input layer is of the size
% of the number of secondary features and the output layer of the size of the number of 
% classes. The linear NN might be trained analytically by using the pseudoinverse method.
% For M classes, N secondary features and P examples in the training data the classification
% problem might be written as follows:
% 
% 		Y = W * X        with 
%
% - W (M*N) as weight matrix of the NN 
% - Y (M*P) as target matrix, each column being the target vector for one example 
% - X (N*P) as feature matrix, each column containing the secondary feature vector of one example 
% 
%     W = Y * X' * inv(X * X')  yields the optimal weight matrix (lowest RMS error).
% 
% The computationally easy way of training the linear NN allows for an automatic optimization
% of the most suitable secondary feature templates. Using the 'substitution rule' implemented
% here, first a number of secondary feature templates is chosen randomly under given constraints
% regarding the position of the smaller window, the extend of the larger window and the
% distance between the two windows in both, frequency and time dimension. The NN is then trained
% on full X and X (without row or feature n) using the pseudoinverse method. The difference in RMS error 
% between the classifiction performance based on X (without row or feature n) and full X (all features)
% yields the relevancies of the secondary feature templates for the given classification task.
% Each Iteration the least relevant secondary feature template is then discarded and 
% replaced by a randomly chosen replacement given the above mentioned contraints.
% 
% Literature:
% - Gram, T.: "Worterkennung mit einem knstlichen Neuronalen Netzwerk", PhD Thesis, 
%   Universitt Gttingen, 1992.
% - Gram, T., and Strube, H.-W.: "Recognition of isolated words
% based on psychoacoustics and neurobiology", Speech Communication
% 9 (1990), pp. 35-40
% - Kleinschmidt, M. and Hohmann, V.: "Perzeptive Vorverarbeitung
% und automatische Selektion sekundrer Merkmale zur robusten
% Spracherkennung", Fortschritte der Akustik - DAGA 2000,
% Oldenburg, 2000. See pdf in the 'papers' subdirectory!
% - Meyer, B. and Kleinschmidt, M.: "Robust Speech-Recognition based 
%  on Localized Spectro-Temporal Features"
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%
% 2. Overview of MATLAB scripts and functions
%
% - run_ffnn.m : Main script for defining, training and testing FFNN 
%
% a) 1st level scripts and functions:
%
%   - load_zifkom.m     : load training and testing data from file 
%                         from ieee32 format preprocessed ZIFKOM data
%   - load_features.m   : load set of secondary feature templates
%   - gen_features.m    : generate set of secondary feature templates
%   - extract_sigmapi.m : extract secondary features from primary features 
%   - train_ffnn.m      : train FFNN on given dataset by using substitution rule
%   - recognize_ffnn.m  : use FFNN to perform recognition experiments on test data
%   - feature_analysis.m: visual analysis of feature set
%   - test_gen_feat.m   : test feature generation
%
% b) 2nd level scripts and functions:
%   - readfile.m			: read primary features from IEEE32 format file
%   - neumerk.m         : generate one secondary feature template randomly given contraints
%   - neumerk2.m        : generate one secondary feature template randomly given contraints
%  			  avoiding identical features by checking other templates
%   - OpenWindow.m  : open and positionate windows for visual training and testing
%   - makeziel.m        : build target matrix
%   - trainer.m         : train net using pseudoinverse method
%   - evaluate.m        : calculate relevances of secondary feature templates
%   - classify.m        : classify, returns recognition rate and confusion matrix
%   - fehler.m          : calculate RMS error of classification task
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 3. Annotations
%
%% a) This MATLAB implementation does not cover major parts of the work of Tino Gram,
%    namely other feature selection rules and his proposals to reduce the 
%    computational effort significantly by taking advantage of intriguing mathematical 
%    'shortcuts'.
%
% b) Changes from v1.0 to v1.2 (Michael Kleinschmidt)
%    -  discarded t0min and f0min, now feature parameters equally distributed
%    -  added feature_analysis.m : visual analysis of feature set
%    -  test_gen_feat.m   : test feature generation
%    Changes from v3.5.5 to v4.0 (Michael Kleinschmidt)
%    - incorporate AdaBoost algorithm
%    Changes from v4.0 to v4.1 (Bernd Meyer)
%    - added selection of different envelope types
%    - added option for different frame stepping (for compatability with
%      Philips recognizer ASPIRIN)
%    - code cleanup and translation
%    - rewrote gabor.m to resemble the c-version 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 5. Copyright Notice and Disclaimer
%
% Permission to use and copy this software is restricted to 
% people and institutions directly authorized by the authors.
%
% ***************COPYRIGHT NOTICE ******************************************* 
%
% =========================================================================== 
%    Copyright (c) AG Medizinische Physik, University of Oldenburg, 2000  
% =========================================================================== 
%                                                                             
% Permission to use and copy this software is restricted to people and        
% applications that were directly authorized by the University Oldenburg, AG 
% Medizinische Physik, c/o Prof. Dr. Dr. B. Kollmeier.     
%                                                                        
% Permission to modify and distribute this software is not granted to anyone. 
%                                                                             
% Anyone wanting to incorporate all or part of this software in a commercial  
% product must obtain a license from the University Oldenburg, AG Medizinische 
% Physik, c/o Prof. Dr. Dr. B. Kollmeier.
%                                                                             
% The authors and the University Oldenburg, AG Medizinische Physik make no 
% representations about the suitability of this software for any purpose. It is 
% provided "as is" without expressed or implied warranty.                                        %      
%                                                                             
% THE AUTHORS AND THE UNIVERSITY OLDENBURG, AG MEDIZINISCHE PHYSIK DISCLAIM ALL
% WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
% MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE AUTHORS OR THE UNIVERSITY 
% OLDENBURG, AG MEDIZINISCHE PHYSIK BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
% CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, 
% DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER 
% TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
% OF THIS SOFTWARE.                                                             
%                                                                 
% Authors:                                                                    
%                                                                             
% Michael Kleinschmidt <michael@medi.physik.uni-oldenburg.de>   
% Bernd Meyer <bernd.meyer@uni-oldenburg.de>             
%                                                          
% Address:                                                                    
%                                                                             
% AG Medizinische Physik                                                      
% University of Oldenburg                                                    
% D-26111 Oldenburg                                                          
% Germany                                                                    
%                                                                             
% Tel: ++49 441 798 3344
% Fax: ++49 441 798 3902                                                     
% WWW: http://medi.uni-oldenburg.de               
%                                                                             
% ************************ END COPYRIGHT NOTE *********************************/




%%-----------------------------------------------------------------------------
%%
%%   Copyright (C) 2000-2004   
%%   Medizinische Physik,
%%   Universitaet Oldenburg, Germany
%%   http://www.physik.uni-oldenburg.de/docs/medi
%%
%%   Permission to use, copy, and distribute this software/file and its
%%   documentation for any purpose without permission by UNIVERSITAET OLDENBURG
%%   is not granted.
%%   
%%   Permission to use this software for academic purposes is generally
%%   granted.
%%
%%   Permission to modify the software is granted, but not the right to
%%   distribute the modified code.
%%
%%   This software is provided "as is" without expressed or implied warranty.
%%
%%   Author:    Michael Kleinschmidt
%%   Co-Author: Bernd Meyer 
%%
%%   Please contact Bernd Meyer (bernd.meyer@uni-oldenburg.de) for 
%%   questions regarding this release.
%%-----------------------------------------------------------------------------
